######################################
#        serial_data_server.pl       #
# Copyright 2012 Bueno systems, Inc. #
#  See end of file for terms of use. #
######################################

use strict;

use Win32::SerialPort;
use HTTP::Daemon;
use HTTP::Status;
use HTTP::Response;
use Time::HiRes qw/sleep/;
require URI;
require Carp::Heavy;

$| = 1;
my %arg;

print "\n*** PhiPi Serial-Network-Serial Server ***\n\n";

foreach my $arg (@ARGV) {
	if ($arg =~ m/([^=]+)=([^=]+)/) {
		$arg{$1} = $2
	} elsif ($arg eq 'help') {
		help();
	} else {
		error("Malformed argument: $arg");
	}
}
error("Missing or invalid comm parameter.") unless (defined $arg{comm} && $arg{comm} =~ m/COM\d{1,3}/i);
error("Missing or invalid baud rate") unless defined $arg{baud} && $arg{baud} =~ m/^(110|150|300|600|1200|2400|4800|9600|19200|31250|38400|57600|115200)$/;
if (defined $arg{port}) {
	error("Invalid IP port number.") unless $arg{port} =~ m/^\d{1,5}$/
} else {
	$arg{port} = 12345
}
if (defined $arg{echo}) {
	error("Invalid Echo parameter") unless $arg{echo} =~ m/^(tx|rx|both)$/
} else {
	$arg{echo} = ''
}

my $daemon;

if ($daemon = HTTP::Daemon->new(LocalPort => $arg{port})) {
	my $url = $daemon->url;
	$url =~ s/\/$//;
	print "Please connect using URL: ", $url, "\n"
} else {
	print "Can't open IP port $arg{port}. Please pick another.";
	exit
}

my $Port;

print "Connecting to $arg{comm}...\n";

my $portname = "\\\\.\\" . $arg{comm};
if ($Port = new Win32::SerialPort($portname)
	and	$Port->baudrate($arg{baud})
	and $Port->parity('none')
	and $Port->databits(8)
	and $Port->stopbits(2)
	and $Port->handshake('none')
	and $Port->buffers(4096,4096)
	and $Port->binary(1)
	and $Port->datatype('raw')
	and $Port->dtr_active('N')
	and $Port->write_settings)
{
	print "Connected to $arg{comm}.\n"
} else {
	print "Connection to $arg{comm} failed.\n";
	exit
}

print "Listening for network connections...\n";

while (my $connection = $daemon->accept) {
	while (my $request = $connection->get_request) {
		if ($request->method eq 'GET' and $request = $request->url) {
			if ($request->path eq '/data') {
				put_sio($request);
				my $response = HTTP::Response->new(200);				
				$response->content(get_sio());
        $response->header("Content-Type" => "text/plain");
        $connection->send_response($response);
			} elsif ($request->path eq '/enq') {
				$connection->send_response()
			}
		} else {
			print "Error\n";
			$connection->send_error(RC_FORBIDDEN)
		}
	}
	$connection->close;
	undef($connection)
}

sub get_sio {
	my $data_out = '';
	my $data = '';
	my $inp;
	do {	
		sleep 0.1;
		$inp = $Port->input;
		$data .= $inp;
	} while $inp ne '';
	echo($data) if $arg{echo} =~ m/tx|both/;
	return $data
}

sub put_sio {
	my $data = shift;
	$data =~ s/^\/data\?//;
	$data =~ s/%([\dA-F]{2})/chr(hex($1))/ge;
	echo($data) if $arg{echo} =~/rx|both/;
	$Port->write($data);
}

sub error {
	print shift(), "\n\nSee below for correct usage.\n\n";
	help()
}

sub help {
	print "Synopsis (arguments in [brackets] optional):\n\n";
	print "  serial_server.exe comm=<comm port> baud=<baud rate> [port=<port num.>]\n";
	print "    [echo=<direction>]\n\n";
	print "    <comm port> = COM1 to COM999\n";
	print "    <baud rate> = any of 110, 150, 300, 600, 1200, 2400, 4800,\n";
	print "                  9600, 19200, 31250, 38400, 57600, or 115200\n";
	print "    <port num.> = IP port specification. Default is 12345 if unspecified.\n\n";
	print "    <direction> = tx:   Echo data to client.\n";
	print "                  rx:   Echo data from client.\n";
	print "                  both: Echo data to and from host (jumbled together;\n";
	print "                        might be a bit messy).\n";
	print "      When echoing, CR is converted to CRLF, and other control characters are\n";
	print "      displayed in <hex>.\n\n";
	print "Arguments are separated by spaces with NO SPACES surrounding equal signs (=).\n\n";
	exit
}

sub echo {
	my $data = shift;
	$data =~ s/([\x00-\x09\x0b\x0c\x0e-\x1f])/sprintf('<%2.2X>',ord($1))/ge;
	$data =~ s/\r/\r\n/g;
	print $data
}
__END__
+--------------------------------------------------------------------------------------+
                            TERMS OF USE: MIT License                                                                                             
+--------------------------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining a copy of this  
software and associated documentation files (the "Software"), to deal in the Software 
without restriction, including without limitation the rights to use, copy, modify,    
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to    
permit persons to whom the Software is furnished to do so, subject to the following   
conditions:                                                                           
                                                                                      
The above copyright notice and this permission notice shall be included in all copies 
or substantial portions of the Software.                                              
                                                                                      
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,   
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A         
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT    
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF  
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE  
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                                         
+--------------------------------------------------------------------------------------+
	
	